/*
MIDI Sequencer V.1.0
(C)2007 by Lindenberg Electronics
All Rights Reserved
*/

#include "main.h"
#include <avr/io.h>
#include <stdint.h>
#include <util/delay.h>
#include "hd44780.h"
#include "uart.h"


/* PORT defines */
#define _KEYPORT	PORTD
#define _KEYOUT		DDRD	

/* keys from keyped */
#define _KEY_LEFT	0b00010000
#define _KEY_RIGHT	0b00001000
#define _KEY_UP		0b00000100

/* main loop msg */
#define _MSG_RESET		0x00
#define _MSG_NOTEPLAY	0x05

/* notes */
#define _N_BLANK		0xFF
#define _N_C			0x00
#define _N_CM			0x01
#define _N_D			0x02
#define _N_DM			0x03
#define _N_E			0x04
#define _N_F			0x05
#define _N_FM			0x06
#define _N_G			0x07
#define _N_GM			0x08
#define _N_A			0x09
#define _N_AM			0x0A
#define _N_H			0x0B	


/* generel MIDI commands */
#define _MIDI_NOTEON	0x91
#define _MIDI_NOTEOFF	0x81
#define _MIDI_VELOCITY	0x40
#define _MIDI_C3		0x30

/* globale variablen */
volatile uint8_t _msg = _MSG_RESET;
volatile uint8_t _nc = 0;
volatile uint16_t time_interval;

void enable_timer(uint8_t bpm)
{
	TCCR1B = 0b00000100;				// Prescaler 256, mit 64 kann man keine
										// niedrigen BPMs erzeugen
//	time_interval = 31250;				// 2 Hz, 500ms, 120 bpm
//	time_interval = 6944;				// 9 Hz, 111ms, 540 bpm!!!
	time_interval = 60*F_CPU/256/bpm;
	OCR1A = time_interval;	
	TIMSK |= (1<<OCIE1A);
}

int main(void)
{
	uint8_t notes[15] ={_N_C, _N_D, _N_E, _N_F, _N_G, _N_A, _N_H, _N_C, 
						_N_C, _N_C, _N_C, _N_C, _N_C, _N_C, _N_C};

	uint8_t last_note = _N_BLANK;

	DDRD = 0xFF;
	PORTD = 0x00;

	lcd_init();
	lcd_clear();
	lcd_home();

	uart_enable();

	lcd_putstr(" MIDI-Sequencer");
	lcd_setline(2);
	lcd_putstr(" (C)2007 by LE");

	enable_timer(135);			// Erst alles initialisieren, dann Interrupts freigeben

	sei();
	

	while(1) {		
		switch(_msg) {
			case _MSG_NOTEPLAY: 
				if (notes[_nc] != _N_BLANK) {
					uart_putc( _MIDI_NOTEOFF );
					uart_putc( _MIDI_C3 + last_note );
					uart_putc( _MIDI_VELOCITY );
		
					uart_putc( _MIDI_NOTEON );
					uart_putc( _MIDI_C3 + notes[_nc] );
					uart_putc( _MIDI_VELOCITY );

					last_note = notes[_nc];
				}
				_msg = _MSG_RESET;
				break;							// sicherheithalber, vergisst man gern beim erweitern des SWITCH

			default: break;
		}
	}

	return 0;
}


ISR(TIMER1_COMPA_vect)
{
	OCR1A += time_interval;

	PORTD ^= 0x80;						// debug, toggle PD7
			
	if(_nc<15) _nc++; else _nc=0;
	_msg = _MSG_NOTEPLAY;
}





